@@ -18,8 +18,7 @@ minimum_python_37 = pytest.mark.skipif(
 )
 
 
-def otel_attributes_to_dict(otel_attrs):
-    # type: (Mapping[str, Any]) -> Mapping[str, Any]
+def otel_attributes_to_dict(otel_attrs: "Mapping[str, Any]") -> "Mapping[str, Any]":
     def _convert_attr(attr):
         # type: (Mapping[str, Union[str, float, bool]]) -> Any
         if attr["type"] == "boolean":
@@ -39,12 +38,12 @@ def otel_attributes_to_dict(otel_attrs):
 
 
 def envelopes_to_logs(envelopes: List[Envelope]) -> List[Log]:
-    res = []  # type: List[Log]
+    res: "List[Log]" = []
     for envelope in envelopes:
         for item in envelope.items:
             if item.type == "log":
                 for log_json in item.payload.json["items"]:
-                    log = {
+                    log: "Log" = {
                         "severity_text": log_json["attributes"]["sentry.severity_text"][
                             "value"
                         ],
@@ -55,7 +54,8 @@ def envelopes_to_logs(envelopes: List[Envelope]) -> List[Log]:
                         "attributes": otel_attributes_to_dict(log_json["attributes"]),
                         "time_unix_nano": int(float(log_json["timestamp"]) * 1e9),
                         "trace_id": log_json["trace_id"],
-                    }  # type: Log
+                        "span_id": log_json["span_id"],
+                    }
                     res.append(log)
     return res
 
@@ -142,6 +142,7 @@ def test_logs_before_send_log(sentry_init, capture_envelopes):
             "attributes",
             "time_unix_nano",
             "trace_id",
+            "span_id",
         }
 
         if record["severity_text"] in ["fatal", "error"]:
@@ -319,7 +320,9 @@ def test_logs_tied_to_transactions(sentry_init, capture_envelopes):
 
     get_client().flush()
     logs = envelopes_to_logs(envelopes)
-    assert logs[0]["attributes"]["sentry.trace.parent_span_id"] == trx.span_id
+
+    assert "span_id" in logs[0]
+    assert logs[0]["span_id"] == trx.span_id
 
 
 @minimum_python_37
@@ -336,7 +339,7 @@ def test_logs_tied_to_spans(sentry_init, capture_envelopes):
 
     get_client().flush()
     logs = envelopes_to_logs(envelopes)
-    assert logs[0]["attributes"]["sentry.trace.parent_span_id"] == span.span_id
+    assert logs[0]["span_id"] == span.span_id
 
 
 def test_auto_flush_logs_after_100(sentry_init, capture_envelopes):
@@ -359,8 +362,8 @@ def test_auto_flush_logs_after_100(sentry_init, capture_envelopes):
 
 
 def test_log_user_attributes(sentry_init, capture_envelopes):
-    """User attributes are sent if enable_logs is True."""
-    sentry_init(enable_logs=True)
+    """User attributes are sent if enable_logs is True and send_default_pii is True."""
+    sentry_init(enable_logs=True, send_default_pii=True)
 
     sentry_sdk.set_user({"id": "1", "email": "test@example.com", "username": "test"})
     envelopes = capture_envelopes()
@@ -381,6 +384,26 @@ def test_log_user_attributes(sentry_init, capture_envelopes):
     }
 
 
+def test_log_no_user_attributes_if_no_pii(sentry_init, capture_envelopes):
+    """User attributes are not if PII sending is off."""
+    sentry_init(enable_logs=True, send_default_pii=False)
+
+    sentry_sdk.set_user({"id": "1", "email": "test@example.com", "username": "test"})
+    envelopes = capture_envelopes()
+
+    python_logger = logging.Logger("test-logger")
+    python_logger.warning("Hello, world!")
+
+    get_client().flush()
+
+    logs = envelopes_to_logs(envelopes)
+    (log,) = logs
+
+    assert "user.id" not in log["attributes"]
+    assert "user.email" not in log["attributes"]
+    assert "user.name" not in log["attributes"]
+
+
 @minimum_python_37
 def test_auto_flush_logs_after_5s(sentry_init, capture_envelopes):
     """
@@ -491,6 +514,7 @@ def test_batcher_drops_logs(sentry_init, monkeypatch):
                     "level": "info",
                     "timestamp": mock.ANY,
                     "trace_id": mock.ANY,
+                    "span_id": mock.ANY,
                     "attributes": {
                         "sentry.environment": {
                             "type": "string",
@@ -516,10 +540,6 @@ def test_batcher_drops_logs(sentry_init, monkeypatch):
                             "type": "string",
                             "value": "info",
                         },
-                        "sentry.trace.parent_span_id": {
-                            "type": "string",
-                            "value": mock.ANY,
-                        },
                         "server.address": {
                             "type": "string",
                             "value": "test-server",
