@@ -10,14 +10,13 @@ from sentry_sdk.types import Metric
 from sentry_sdk.consts import SPANDATA, VERSION
 
 
-def envelopes_to_metrics(envelopes):
-    # type: (List[Envelope]) -> List[Metric]
+def envelopes_to_metrics(envelopes: "List[Envelope]") -> "List[Metric]":
     res = []  # type: List[Metric]
     for envelope in envelopes:
         for item in envelope.items:
             if item.type == "trace_metric":
                 for metric_json in item.payload.json["items"]:
-                    metric = {
+                    metric: "Metric" = {
                         "timestamp": metric_json["timestamp"],
                         "trace_id": metric_json["trace_id"],
                         "span_id": metric_json.get("span_id"),
@@ -29,7 +28,7 @@ def envelopes_to_metrics(envelopes):
                             k: v["value"]
                             for (k, v) in metric_json["attributes"].items()
                         },
-                    }  # type: Metric
+                    }
                     res.append(metric)
     return res
 
@@ -117,7 +116,7 @@ def test_metrics_with_attributes(sentry_init, capture_envelopes):
 
 
 def test_metrics_with_user(sentry_init, capture_envelopes):
-    sentry_init()
+    sentry_init(send_default_pii=True)
     envelopes = capture_envelopes()
 
     sentry_sdk.set_user(
@@ -135,6 +134,25 @@ def test_metrics_with_user(sentry_init, capture_envelopes):
     assert metrics[0]["attributes"]["user.name"] == "testuser"
 
 
+def test_metrics_no_user_if_pii_off(sentry_init, capture_envelopes):
+    sentry_init(send_default_pii=False)
+    envelopes = capture_envelopes()
+
+    sentry_sdk.set_user(
+        {"id": "user-123", "email": "test@example.com", "username": "testuser"}
+    )
+    sentry_sdk.metrics.count("test.user.counter", 1)
+
+    get_client().flush()
+
+    metrics = envelopes_to_metrics(envelopes)
+    assert len(metrics) == 1
+
+    assert "user.id" not in metrics[0]["attributes"]
+    assert "user.email" not in metrics[0]["attributes"]
+    assert "user.name" not in metrics[0]["attributes"]
+
+
 def test_metrics_with_span(sentry_init, capture_envelopes):
     sentry_init(traces_sample_rate=1.0)
     envelopes = capture_envelopes()
